﻿//
// Copyright (C) Pillars Inc. - All Rights Reserved
//
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data.Entity;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Xml.Linq;

using CsvHelper;
using CsvHelper.Configuration.Attributes;

using Microsoft.AspNet.SignalR;

using Rock;
using Rock.Attribute;
using Rock.Data;
using Rock.Model;
using Rock.Security;
using Rock.Web.Cache;
using Rock.Web.UI.Controls;

namespace RockWeb.Plugins.rocks_pillars.Finance
{
    /// <summary>
    /// Template block for developers to use to start a new block.
    /// </summary>
    [DisplayName( "My Well Transaction Import" )]
    [Category( "Pillars > Finance" )]
    [Description( "Block for importing My Well transactions." )]

    [LinkedPage( "Batch Detail Page", "The page used to display details of a batch.", false, "", "", 0 )]
    [AccountField( "Default Account", "The account to use if transaction cannot be matched by Fund Code or Name", true, "", "", 2)]
    [DefinedValueField(Rock.SystemGuid.DefinedType.FINANCIAL_SOURCE_TYPE, "Default Source", "", false, false, "", "", 3)]
    public partial class MyWellTransactionImport : Rock.Web.UI.RockBlock
    {
        #region Fields

        private readonly string _foreignKey = "MyWell";
        private readonly IHubContext _hubContext = GlobalHost.ConnectionManager.GetHubContext<RockMessageHub>();
        private readonly int _transactionTypeContributionId = Rock.Web.Cache.DefinedValueCache.Get( Rock.SystemGuid.DefinedValue.TRANSACTION_TYPE_CONTRIBUTION.AsGuid() ).Id;
        private readonly int _currencyTypeAch = DefinedValueCache.Get( Rock.SystemGuid.DefinedValue.CURRENCY_TYPE_ACH.AsGuid() ).Id;
        private readonly int _currencyTypeCC = DefinedValueCache.Get( Rock.SystemGuid.DefinedValue.CURRENCY_TYPE_CREDIT_CARD.AsGuid() ).Id;
        private readonly DefinedTypeCache _ccTypes = DefinedTypeCache.Get( Rock.SystemGuid.DefinedType.FINANCIAL_CREDIT_CARD_TYPE );

        protected string signalREventName = "myWellTransactionImport";
        private int? _binaryFileId = null;
        private decimal _totalAmount = 0.0M;

        #endregion

        #region Base Control Methods

        /// <summary>
        /// Restores the view-state information from a previous user control request that was saved by the <see cref="M:System.Web.UI.UserControl.SaveViewState" /> method.
        /// </summary>
        /// <param name="savedState">An <see cref="T:System.Object" /> that represents the user control state to be restored.</param>
        protected override void LoadViewState( object savedState )
        {
            base.LoadViewState( savedState );
            _binaryFileId = ViewState["BinaryFileId"] as int?;
        }

        /// <summary>
        /// Raises the <see cref="E:System.Web.UI.Control.Init" /> event.
        /// </summary>
        /// <param name="e">An <see cref="T:System.EventArgs" /> object that contains the event data.</param>
        protected override void OnInit( EventArgs e )
        {
            base.OnInit( e );

            signalREventName = string.Format( "myWellTransactionImport_{0}_{1}", this.BlockId, Session.SessionID );

            RockPage.AddScriptLink( "~/Scripts/jquery.signalR-2.2.0.min.js", fingerprint: false );

            // this event gets fired after block settings are updated. it's nice to repaint the screen if these settings would alter it
            this.BlockUpdated += Block_BlockUpdated;
            this.AddConfigurationUpdateTrigger( upnlContent );

            dvpSource.DefinedTypeId = DefinedTypeCache.Get( Rock.SystemGuid.DefinedType.FINANCIAL_SOURCE_TYPE ).Id;
        }

        /// <summary>
        /// Raises the <see cref="E:System.Web.UI.Control.Load" /> event.
        /// </summary>
        /// <param name="e">The <see cref="T:System.EventArgs" /> object that contains the event data.</param>
        protected override void OnLoad( EventArgs e )
        {
            base.OnLoad( e );

            // Set timeout for up to 15 minutes 
            Server.ScriptTimeout = 900;
            ScriptManager.GetCurrent( Page ).AsyncPostBackTimeout = 900;

            nbMessage.Visible = false;

            if ( !Page.IsPostBack )
            {
                using ( var rockContext = new RockContext() )
                {
                    // Load dropdown list with all of the open batches
                    var batches = new FinancialBatchService( rockContext )
                        .Queryable().AsNoTracking()
                        .Where( b =>
                            b.Status != BatchStatus.Closed &&
                            b.BatchStartDateTime.HasValue )
                        .OrderByDescending( b => b.BatchStartDateTime )
                        .ThenBy( b => b.Name )
                        .Select( b => new
                        {
                            b.Id,
                            b.Name,
                            b.BatchStartDateTime
                        } )
                        .ToList()
                        .Select( b => new
                        {
                            b.Id,
                            Name = string.Format( "{0} ({1})", b.Name, b.BatchStartDateTime.Value.ToShortDateString() )
                        } )
                        .ToList();

                    ddlBatch.DataSource = batches;
                    ddlBatch.DataBind();
                    ddlBatch.Items.Insert( 0, new ListItem() );

                }

                dpBatchDate.SelectedDate = RockDateTime.Today;

                Guid? defaultSourceGuid = GetAttributeValue( "DefaultSource" ).AsGuidOrNull();
                if ( defaultSourceGuid.HasValue )
                {
                    dvpSource.SetValue( DefinedValueCache.Get( defaultSourceGuid.Value ).Id );
                }

                ShowEntry();
            }
        }

        /// <summary>
        /// Saves any user control view-state changes that have occurred since the last page postback.
        /// </summary>
        /// <returns>
        /// Returns the user control's current view state. If there is no view state associated with the control, it returns null.
        /// </returns>
        protected override object SaveViewState()
        {
            ViewState["BinaryFileId"] = _binaryFileId;
            return base.SaveViewState();
        }

        #endregion

        #region Events

        /// <summary>
        /// Handles the BlockUpdated event of the control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void Block_BlockUpdated( object sender, EventArgs e )
        {
            ShowBatchOptions();
        }

        /// <summary>
        /// Handles the FileUploaded event of the fuTxnFile control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void fuTxnFile_FileUploaded( object sender, EventArgs e )
        {
            _binaryFileId = fuTxnFile.BinaryFileId;

            IsFileValid();
        }

        /// <summary>
        /// Handles the SelectedIndexChanged event of the ddlBatch control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void ddlBatch_SelectedIndexChanged( object sender, EventArgs e )
        {
            ShowBatchOptions();
        }

        /// <summary>
        /// Handles the Click event of the btnImport control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void btnImport_Click( object sender, EventArgs e )
        {
            if ( IsFileValid() )
            {
                ShowConfirmation();
            }
        }

        /// <summary>
        /// Handles the Click event of the btnConfirm control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void btnConfirm_Click( object sender, EventArgs e )
        {
            // send signalR message to start progress indicator
            int progress = 0;
            _hubContext.Clients.All.receiveNotification( signalREventName, "0" );

            List<MyWellTxn> txns = null;
            int? txnCount = null;
            int? batchId = null;

            using ( var rockContext = new RockContext() )
            {
                // Get the XML
                txns = GetTxnFile();

                // Get the number of transactions
                if ( txns != null )
                {
                    txnCount = txns.Count();
                }

                if ( txns != null && txnCount.HasValue && txnCount.Value > 0 )
                {
                    var batchService = new FinancialBatchService( rockContext );
                    FinancialBatch batch = null;

                    // Load (or create) the batch
                    batchId = ddlBatch.SelectedValueAsInt();
                    if ( batchId.HasValue )
                    {
                        batch = batchService.Get( batchId.Value );
                    }

                    string batchName;
                    if ( batch == null )
                    {
                        batchName = tbBatchName.Text.IsNotNullOrWhiteSpace() ? tbBatchName.Text : "My Well Batch";

                        batch = new FinancialBatch();
                        batch.Guid = Guid.NewGuid();
                        batch.Name = batchName;
                        batch.Status = BatchStatus.Pending;
                        batch.BatchStartDateTime = dpBatchDate.SelectedDate ?? RockDateTime.Today;
                        batch.BatchEndDateTime = batch.BatchStartDateTime.Value.AddDays( 1 );
                        batch.CampusId = cpCampus.SelectedCampusId;
                        batch.ControlAmount = 0;
                        batchService.Add( batch );

                        rockContext.SaveChanges();

                        batchId = batch.Id;
                    }
                    else
                    {
                        batchName = batch.Name;
                    }
                }
            }

            // Initialize the status variables
            int matchCount = 0;
            int unmatchCount = 0;
            int errorCount = 0;

            var allErrorMessages = new List<string>();

            // Process each transaction
            foreach ( var txn in txns )
            {
                var errorMessages = new List<string>();

                var status = ProcessTransaction( txn, batchId, out errorMessages );

                switch ( status )
                {
                    case ProcessStatus.Matched: matchCount++; break;
                    case ProcessStatus.Unmatched: unmatchCount++; break;
                    case ProcessStatus.Error: errorCount++; break;
                }

                allErrorMessages.AddRange( errorMessages );

                // Update progress using signalR
                progress++;
                int percentage = ( progress * 100 ) / txnCount.Value;
                _hubContext.Clients.All.receiveNotification( signalREventName, percentage.ToString( "N0" ) );
            }

            // update success message to indicate the txns that were updated
            StringBuilder sb = new StringBuilder();
            sb.AppendFormat( "<li>{0:N0} {1} processed.</li>", txnCount.Value, "transaction".PluralizeIf( txnCount.Value > 1 ) );
            sb.AppendFormat( "<li>{0:N0} {1} matched to an existing person.</li>", matchCount,
                ( matchCount == 1 ? "transaction was" : "transactions were" ) );
            sb.AppendFormat( "<li>{0:N0} {1} NOT matched to an existing person.</li>", unmatchCount,
                ( unmatchCount == 1 ? "transaction was" : "transactions were" ) );
            if ( errorCount > 0 )
            {
                sb.AppendFormat( "<li>{0:N0} {1} NOT imported due to error during import (see errors below).</li>", errorCount,
                    ( errorCount == 1 ? "transaction was" : "transactions were" ) );
            }

            using ( var rockContext = new RockContext() )
            {
                var batch = new FinancialBatchService( rockContext ).Get( batchId.Value );
                if ( batch != null )
                {
                    // update batch control amount
                    batch.ControlAmount += _totalAmount;
                    rockContext.SaveChanges();

                    // Add link to batch
                    var qryParam = new Dictionary<string, string>();
                    qryParam.Add( "batchId", batchId.ToString() );
                    string batchUrl = LinkedPageUrl( "BatchDetailPage", qryParam );
                    string batchLink = string.Format( "<a href='{0}'>{1}</a>", batchUrl, batch.Name );

                    int totalTransactions = matchCount + unmatchCount;
                    string summaryformat = totalTransactions == 1 ?
                        "<li>{0} transaction of {1} was added to the {2} batch.</li>" :
                        "<li>{0} transactions totaling {1} were added to the {2} batch</li>";
                    sb.AppendFormat( summaryformat, totalTransactions.ToString( "N0" ), _totalAmount.FormatAsCurrency(), batchLink );
                }
            }

            nbSuccess.Text = string.Format( "<ul>{0}</ul>", sb.ToString() );

            // Display any errors that occurred
            if ( allErrorMessages.Any() )
            {
                StringBuilder sbErrors = new StringBuilder();
                foreach ( var errorMsg in allErrorMessages )
                {
                    sbErrors.AppendFormat( "<li>{0}</li>", errorMsg );
                }

                nbErrors.Text = string.Format( "<ul>{0}</ul>", sbErrors.ToString() );
                nbErrors.Visible = true;
            }
            else
            {
                nbErrors.Visible = false;
            }

            ShowResults();

        }

        /// <summary>
        /// Handles the Click event of the btnCancelConfirm control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void btnCancelConfirm_Click( object sender, EventArgs e )
        {
            ShowEntry();
        }

        /// <summary>
        /// Handles the Click event of the btnImportAnother control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void btnImportAnother_Click( object sender, EventArgs e )
        {
            fuTxnFile.BinaryFileId = null;
            ddlBatch.SetValue( "" );

            ShowEntry();
        }

        #endregion

        #region Methods

        /// <summary>
        /// Shows the entry.
        /// </summary>
        private void ShowEntry()
        {
            pnlEntry.Visible = true;
            pnlConfirm.Visible = false;
            pnlResults.Visible = false;

            ShowBatchOptions();
        }

        /// <summary>
        /// Shows the batch options.
        /// </summary>
        private void ShowBatchOptions()
        {
            bool newBatch = !ddlBatch.SelectedValueAsInt().HasValue;

            tbBatchName.Visible = newBatch;
            dpBatchDate.Visible = newBatch;
            cpCampus.Visible = newBatch;

            tbBatchName.Required = newBatch;
            dpBatchDate.Required = newBatch;
        }

        /// <summary>
        /// Shows the confirmation.
        /// </summary>
        private void ShowConfirmation()
        {
            // Get the number of transactions in the XML file
            int? txnCount = null;
            var myWellFile = GetTxnFile();
            if ( myWellFile != null )
            {
                txnCount = myWellFile.Count();
            }

            if ( txnCount.HasValue )
            {
                string batchName = "a new batch";
                var binaryFile = new BinaryFileService( new RockContext() ).Get( _binaryFileId.Value );
                if ( binaryFile != null )
                {
                    batchName = string.Format( "a new batch (<strong><em>{0}</em></strong>)", Path.GetFileNameWithoutExtension( binaryFile.FileName ) );
                }

                int? batchId = ddlBatch.SelectedValueAsInt();
                if ( batchId.HasValue )
                {
                    using ( var rockContext = new RockContext() )
                    {
                        var batch = new FinancialBatchService( rockContext ).Get( batchId.Value );
                        if ( batch != null )
                        {
                            batchName = string.Format( "the <strong><em>{0} ({1})</em></strong> batch", batch.Name, batch.BatchStartDateTime.Value.ToShortDateString() );
                        }
                    }
                }
                else
                {
                    if ( tbBatchName.Text.IsNotNullOrWhiteSpace() )
                    {
                        batchName = string.Format( "a new batch (<strong><em>{0}</em></strong>)", tbBatchName.Text );
                    }
                }

                lConfirm.Text = string.Format( "This will import <strong>{0:N0}</strong> transactions to {1}. Click <em>Confirm</em> to continue.", txnCount.Value, batchName );

                // Show the confirm/status/result dialog 
                pnlEntry.Visible = false;
                pnlConfirm.Visible = true;
                pnlResults.Visible = false;
            }
        }

        /// <summary>
        /// Shows the results.
        /// </summary>
        private void ShowResults()
        {
            pnlEntry.Visible = false;
            pnlConfirm.Visible = false;
            pnlResults.Visible = true;
        }

        /// <summary>
        /// Processes a transaction.
        /// </summary>
        /// <param name="giftElement">The gift element.</param>
        /// <param name="batch">The batch.</param>
        /// <param name="rockContext">The rock context.</param>
        /// <param name="envelopeAttributeId">The envelope attribute identifier.</param>
        /// <param name="errorMessages">The error messages.</param>
        /// <returns></returns>
        private ProcessStatus ProcessTransaction( MyWellTxn myWellTxn, int? batchId, out List<string> errorMessages )
        {
            errorMessages = new List<string>();

            try
            {
                using ( var rockContext = new RockContext() )
                {

                    var txnService = new FinancialTransactionService( rockContext );

                    string txnKey = string.Format( "{0}|{1}", _foreignKey, myWellTxn.GiftId );

                    var txn = txnService.Queryable()
                        .Where( t => t.ForeignKey == txnKey )
                        .FirstOrDefault();

                    if ( txn == null )
                    {
                        txn = new FinancialTransaction();
                        txn.ForeignKey = txnKey;
                        txn.BatchId = batchId;
                        txn.FinancialPaymentDetail = new FinancialPaymentDetail();
                        txnService.Add( txn );
                    }

                    txn.TransactionTypeValueId = _transactionTypeContributionId;

                    txn.TransactionDateTime = myWellTxn.GiftDate;
                    txn.SettledDate = myWellTxn.SettledDate;
                    txn.SourceTypeValueId = dvpSource.SelectedValueAsInt();
                    txn.Summary = string.Format( "{0} | {1} | {2} | {3} | {4}", myWellTxn.PersonName, myWellTxn.GiftLocation, PhoneNumber.FormattedNumber("1", myWellTxn.PersonPhone), myWellTxn.PersonEmail, myWellTxn.GiftFund );

                    txn.FinancialPaymentDetail.CurrencyTypeValueId = myWellTxn.GiftPaymentType == "ACH" ? _currencyTypeAch : _currencyTypeCC;
                    if ( txn.FinancialPaymentDetail.CurrencyTypeValueId == _currencyTypeCC )
                    {
                        var ccType = _ccTypes.DefinedValues
                            .FirstOrDefault( v => v.Value.Equals( myWellTxn.GiftPaymentType, StringComparison.OrdinalIgnoreCase ) );
                        if ( ccType != null )
                        {
                            txn.FinancialPaymentDetail.CreditCardTypeValueId = ccType.Id;
                        }
                    }

                    txn.MICRStatus = MICRStatus.Success;
                    txn.CheckMicrParts = Encryption.EncryptString( string.Format( "{0}_{1}", _foreignKey, myWellTxn.PersonId ) );
                    if ( txn.AuthorizedPersonAlias == null )
                    {
                        string checkMicrHashed = FinancialPersonBankAccount.EncodeAccountNumber( _foreignKey, myWellTxn.PersonId );
                        var matchedPersons = new FinancialPersonBankAccountService( rockContext )
                            .Queryable()
                            .Where( a => a.AccountNumberSecured == checkMicrHashed )
                            .Select( a => a.PersonAlias.Person )
                            .Distinct();
                        if ( matchedPersons.Count() == 1 )
                        {
                            txn.AuthorizedPersonAliasId = matchedPersons.First().PrimaryAliasId;
                        }
                    }

                    var accountService = new FinancialAccountService( rockContext );
                    FinancialAccount account = accountService.Queryable()
                        .Where( a => a.Name == myWellTxn.GiftFund )
                        .FirstOrDefault();

                    if ( account == null )
                    {
                        var accountGuid = GetAttributeValue( "DefaultAccount" ).AsGuid();
                        account = accountService.Queryable()
                            .Where( a => a.Guid == accountGuid )
                            .FirstOrDefault();
                    }

                    decimal? amount = myWellTxn.GiftAmount;

                    txn.TransactionDetails.Clear();

                    if ( account != null && amount.HasValue )
                    {
                        var txnDetail = new FinancialTransactionDetail();
                        txnDetail.AccountId = account.Id;
                        txnDetail.Amount = amount.Value;
                        txn.TransactionDetails.Add( txnDetail );

                        _totalAmount += amount.Value;
                    }
                    else
                    {
                        errorMessages.Add( string.Format( "Name: {0}; Date: {1}; Account: {2}; Amount: {3}; Error: Invalid Account or Amount",
                            myWellTxn.PersonName, myWellTxn.GiftDate.ToShortDateString(), myWellTxn.GiftFund, myWellTxn.GiftAmount ) );
                    }

                    if ( errorMessages.Any() )
                    {
                        return ProcessStatus.Error;
                    }

                    // Save the transaction 
                    rockContext.SaveChanges();

                    return txn.AuthorizedPersonAliasId.HasValue ? ProcessStatus.Matched : ProcessStatus.Unmatched;
                }

            }

            catch ( Exception ex )
            {
                errorMessages.Add( string.Format( "Name: {0}; Date: {1}; Account: {2}; Amount: {3}; Error: {4}",
                    myWellTxn.PersonName, myWellTxn.GiftDate.ToShortDateString(), myWellTxn.GiftFund, myWellTxn.GiftAmount, ex.Message ) );
                return ProcessStatus.Error;
            }
        }

        /// <summary>
        /// Gets the child element value.
        /// </summary>
        /// <param name="giftElement">The gift element.</param>
        /// <param name="propertyName">Name of the property.</param>
        /// <returns></returns>
        private string GetChildElementValue( XElement giftElement, string propertyName)
        {
            var propElement = giftElement.Element( propertyName );
            if ( propElement != null )
            {
                return propElement.Value;
            }
            return string.Empty;
        }

        /// <summary>
        /// Determines whether the binary file has valid XML.
        /// </summary>
        /// <returns></returns>
        private bool IsFileValid()
        {
            try
            {
                var txnFile = GetTxnFile();
                if ( txnFile == null )
                {
                    ShowError( "Invalid Import File", "Could not read CSV file." );
                    return false;
                }

                if ( !txnFile.Any() )
                {
                    ShowError( "Warning", "The import file does not appear to contain any transactions." );
                    return false;
                }

                return true;
            }
            catch ( Exception ex )
            {
                ShowError( "Invalid Import File", ex.Message );
                return false;
            }
        }

        /// <summary>
        /// Gets the XML.
        /// </summary>
        /// <returns></returns>
        private List<MyWellTxn> GetTxnFile()
        {
            if ( _binaryFileId.HasValue )
            {
                using ( var rockContext = new RockContext() )
                {
                    var binaryFile = new BinaryFileService( rockContext ).Get( _binaryFileId.Value );
                    if ( binaryFile != null )
                    {
                        using ( var stream = binaryFile.ContentStream )
                        {
                            using ( var tr = new StreamReader( stream ) )
                            {
                                CsvReader csvReader = new CsvReader( tr, CultureInfo.InvariantCulture );
                                csvReader.Configuration.HasHeaderRecord = true;
                                return csvReader.GetRecords<MyWellTxn>().ToList();
                            }
                        }
                    }
                }
            }

            return null;
        }

        #region Show Notifications

        /// <summary>
        /// Shows a warning.
        /// </summary>
        /// <param name="title">The title.</param>
        /// <param name="message">The message.</param>
        private void ShowWarning( string title, string message )
        {
            nbMessage.NotificationBoxType = NotificationBoxType.Warning;
            ShowMessage( title, message );
        }

        /// <summary>
        /// Shows a error.
        /// </summary>
        /// <param name="title">The title.</param>
        /// <param name="message">The message.</param>
        private void ShowError( string title, string message )
        {
            nbMessage.NotificationBoxType = NotificationBoxType.Danger;
            ShowMessage( title, message );
        }

        /// <summary>
        /// Shows a message.
        /// </summary>
        /// <param name="title">The title.</param>
        /// <param name="message">The message.</param>
        private void ShowMessage( string title, string message )
        {
            nbMessage.Title = title;
            nbMessage.Text = message;
            nbMessage.Visible = true;
        }

        #endregion

        #endregion

        /// <summary>
        /// Enumeration for tracking transction status
        /// </summary>
        private enum ProcessStatus
        {
            Matched = 0,
            Unmatched = 1,
            Error = 2
        }

    }

    public class MyWellTxn
    {
        [Name( "Gift Id")]
        public string GiftId { get; set; }

        [Name( "Gift Date" )]
        public DateTime GiftDate { get; set; }

        [Name( "Gift Settled Date" )]
        public DateTime? SettledDate { get; set; }

        [Name( "Gift Fund" )]
        public string GiftFund { get; set; }

        [Name( "Gift Location" )]
        public string GiftLocation { get; set; }

        [Name( "Gift Amount" )]
        public decimal? GiftAmount { get; set; }

        [Name( "Gift Payment Type" )]
        public string GiftPaymentType { get; set; }

        [Name( "Gift Is Apple Pay?" )]
        public bool GiftIsApplePay {get; set;}

        [Name( "Gift Status" )]
        public string GiftStatus { get; set; }

        [Name( "Person Id" )]
        public string PersonId { get; set; }

        [Name( "Person Name" )]
        public string PersonName { get; set; }

        [Name( "Person Phone" )]
        public string PersonPhone { get; set; }

        [Name( "Person Email" )]
        public string PersonEmail { get; set; }
    }

}


